import java.awt.Graphics;
import java.awt.Image;

/**
 * A Lunar lander that can fire lasers
 * @author Jim Skrentny modified
 * @author Tim Bahls
 *
 */
public class Defender {

	private String imageName;//the filename of the image
	private int xPosition;//the x position
	private final int yPosition;//the y position
	private int life;//the number of hitpoints left
	
	/**
	 * Constructs a Defender at the given location with the given image
	 * @param startX the x position to start at
	 * @param startY the y position to start at
	 * @param imageName the filename for the image to use
	 */
	public Defender(int startX, int startY, String imageName){
		xPosition=startX;
		yPosition=startY;
		this.imageName=imageName;
		life=3;
	}
	
	/**
	 * Sets the x position to the given position
	 * @param newXPosition
	 */
	public void setXPosition(int newXPosition){
		xPosition=newXPosition;
	}
	
	/**
	 * Return the bullet this Defender is about to fire.
	 * @return The bullet this Defender is about to fire.
	 */
	public Laser fireWeapon(){
		return new Laser(this);
	}
	
	/**
	 * Returns the filename of this Defender's image
	 * @return  the filename of this Defender's image
	 */
	public String getImageName(){
		return this.imageName;
	}
	
	/**
	 * Get the current x coordinate of the center of this Defender.
	 * 
	 * @return The current x coordinate of the center of this Defender.
	 */
	public int getXPosition(){
		return this.xPosition;
	}
	
	/**
	 * Get the current y coordinate of the center of this Defender.
	 * 
	 * @return The current y coordinate of the center of this Defender.
	 */
	public int getYPosition(){
		return this.yPosition;
	}
	
	/**
	 * Returns whether this Defender intersects this Laser
	 * 
	 * @param theLaser
	 * @return true if this Defender intersects this Laser
	 */
	public boolean isHitByLaser(Laser theLaser){
		int defenderSize=SpaceGame.DEFENDER_SIZE/2;
		if( Math.abs(theLaser.getXPosition()-xPosition)<defenderSize){
			if( Math.abs(theLaser.getYPosition()-yPosition)<defenderSize){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Returns true if this Defender is dead.
	 * 
	 * @return true if the Defender is dead
	 */	
	public boolean removeMeFromGame(){
		return life<=0;
	}

	/**
	 * Updates the Object to record that it has been hit by a Laser. This
	 * method is called every time the Defender is struck by a Laser.
	 */
	public void recordHit() {
		life--;
	}

	/**
	 * Paint this Defender onto the given Graphics object
	 * @param g the Graphics object to draw onto
	 */
	public void paint(Graphics g) {
		Image image = ImageCache.getIm(imageName);
		int x = xPosition - SpaceGame.DEFENDER_SIZE / 2;
		int y = yPosition - SpaceGame.DEFENDER_SIZE / 2;
		g.drawImage(image, x, y, null);		
	}
	
}
