import java.awt.*;
import java.awt.event.*;

/**
 * A class for getting a double buffered fix framerate Frame onscreen with mouse
 * and keyboard input onscreen.
 * 
 * The class does not attempt to have proper programming style
 * 
 * @author Tim Bahls
 * 
 */
public class GameFrame extends Frame {

	int SHIFTX = 5;
	int SHIFTY = 20;

	public GameFrame(String s, int x, int y) {
		super(s);
		setSize(x, y);
		init();

	}

	// set this to true to pause the game
	public boolean pause = false;

	// How fast do we run?
	public int maxFPS = 1000;

	// number of frames elapsed
	public int numberOfFrames = 0;

	// where is the mouse, and is it pressed?
	public int mouseX = 0;
	public int mouseY = 0;
	public boolean mouseDown = false;

	// how big is the applet, horizontally and vertically.
	public int hrz, vrt;

	// a background color for your applet. Leave it null if you
	// don't want it.
	public Color clearColor = null;

	// you must call these if you want mouse or keyboard input.
	// call them in your initIt() method.

	public void addMouse() {
		addMouseListener(new MouseControl()); // mouse control
		addMouseMotionListener(new MouseMove()); // mouse control 2
	}

	public void addKeyBoard() {
		addKeyListener(new KeyBoard());
	}

	// /////////////////////////////////////////////////////////
	// Your program should ovverice any of these methods you
	// want to use
	// /////////////////////////////////////////////////////////

	// called once at the beginning.
	public void initIt() {
		// override
	}

	/**
	 * This is called any time the mouse is moved, pressed, released, or
	 * dragged. don't forget that the origin in Java is the top left, not the
	 * bottom left
	 * 
	 * @param mx
	 * @param my
	 * @param pressed
	 * @param released
	 * @param mouseButton
	 */
	public void mouse(int mx, int my, boolean pressed, boolean released,
			int mouseButton) {
		// override
	}

	// this is called any time a key is pressed or released.
	// remember that holding a key down counts as pressing it repeatedly.
	// The parameters c and k decribe the key pressed. You can
	// use either that you find convienient
	public void key(char c, String k, boolean pressed) {
		// override
	}

	// GameApplet calls this over and over, alternating
	// with the paint method
	public void tick() {
		// override
	}

	// GameApplet calls this over and over, alternating
	// with the tick method. Paint everything on this Graphics
	public void paint(Graphics g1) {
		// override
	}

	// /////////////////////////////////////////////////////////
	// You can ignore everything below this line.
	// For internal use only.
	// /////////////////////////////////////////////////////////

	// to prevent a warning, I need this line
	static final long serialVersionUID = 85473697456456L;

	// for graphics
	private Graphics g;
	private Graphics temp;
	private Image tempIm;

	// applets call this automatically when they start
	public void init() {
		setVisible(true);
		hrz = getWidth() - 10;
		vrt = getHeight() - 25;
		tempIm = createImage(hrz, vrt);
		temp = tempIm.getGraphics();
		g = getGraphics();
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		initIt();
	}

	// after init is called, they call start.
	public void start() {
		// launch a thread that calls the tick and paint methods
		// you seem to need a seprate thread for painting stuff,
		// but I never really learned why.
		Thread runner = new Thread() {
			public void run() {
				long lastFrame = System.currentTimeMillis();
				while (true) {
					if (!pause) {
						tick();
					}

					doubleBuffer(g);
					numberOfFrames++;
					try {
						long nextFrame = System.currentTimeMillis();
						Thread.sleep(Math.max(0, lastFrame + 1000 / maxFPS
								- nextFrame));
						lastFrame = nextFrame;
					} catch (Exception e) {
						// do nothing
					}

				}
			}
		};
		runner.start();
	}

	// animation looks terrible without double buffering
	public void doubleBuffer(Graphics g1) {
		if (clearColor != null) {
			temp.setColor(clearColor);
			temp.fillRect(0, 0, hrz, vrt);
		}
		paint(temp);
		g1.drawImage(tempIm, SHIFTX, SHIFTY, null);
	}

	// I don't really like the way mouse and keyboard
	// is done. That's why I translated it to my
	// version
	class MouseControl extends MouseAdapter {

		public void mousePressed(MouseEvent e) {
			mouseDown = true;
			mouseX = e.getX() - SHIFTX;// store click location for use with
			// buttons
			mouseY = e.getY() - SHIFTY;
			mouse(mouseX, mouseY, true, false, e.getButton());
		}

		public void mouseReleased(MouseEvent e) {
			mouseDown = false;
			mouseX = e.getX() - SHIFTX;// store click location for use with
			// buttons
			mouseY = e.getY() - SHIFTY;
			mouse(mouseX, mouseY, false, true, e.getButton());
		}

	}

	class MouseMove extends MouseMotionAdapter {

		public void mouseDragged(MouseEvent e) {
			mouseDown = true;
			mouseX = e.getX() - SHIFTX;// store click location for use with
			// buttons
			mouseY = e.getY() - SHIFTY;
			mouse(mouseX, mouseY, false, false, e.getButton());
		}

		public void mouseMoved(MouseEvent e) {
			mouseDown = false;
			mouseX = e.getX() - SHIFTX;// store click location for use with
			// buttons
			mouseY = e.getY() - SHIFTY;
			mouse(mouseX, mouseY, false, false, e.getButton());
		}

	}

	class KeyBoard extends KeyAdapter {
		public void keyPressed(KeyEvent e) {
			String k = KeyEvent.getKeyText(e.getKeyCode());
			char c = e.getKeyChar();
			key(c, k, true);
		}

		public void keyReleased(KeyEvent e) {
			String k = KeyEvent.getKeyText(e.getKeyCode());
			char c = e.getKeyChar();
			key(c, k, false);
		}

	}

}
