import java.awt.Image;
import java.util.HashMap;

/**
 * A class to hold images and insure they aren't pulled off the hard drive (or
 * the internet) more than once
 * 
 * @author Tim Bahls
 * 
 */
public class ImageCache {

	// the cache itself
	private static HashMap originals = new HashMap();

	// the extension of the particular files we are using
	// defaults to ".png"
	private static String extension = ".png";

	// the Object with the ability to load images
	protected static ImageGetter imageMaster;

	/**
	 * Gets the image described by the fileName. If it isn't cached, we get it
	 * from file and add it to the cache.
	 * 
	 * @param fileName the fileName of the image
	 * @return the Image
	 */
	public static Image getIm(String fileName) {
		if (imageMaster == null) {
			throw new RuntimeException(
					"Cannot get images before setting the imageGetter.");
		}
		Image im = (Image) originals.get(fileName);
		if (im != null) {
			return im;
		}
		im = imageMaster.getIm(fileName + extension);
		originals.put(fileName, im);
		return im;
	}

	/**
	 * Set the ImageGetter so that we may start loading images
	 * 
	 * @param _imageMaster The ImageGetter
	 */
	public static void setImageGetter(ImageGetter _imageMaster) {
		imageMaster = _imageMaster;
	}

	/**
	 * Change the extension to add onto filenames
	 * 
	 * @param ex
	 *            the new extension
	 */
	public static void setExtension(String ex) {
		extension = ex;
	}
}
